@@ -14,6 +14,7 @@ import numpy as np
 from tensorflow import keras
 from tensorflow.keras import backend as K
 import gc
+import pickle
 
 
 class evaluate():
@@ -29,10 +30,6 @@ class evaluate():
 
     **kwargs:**
 
-        xHI: **Bool / default: False**
-            | If True then ``globalemu`` will act as if it is evaluating a
-                neutral fraction history emulator.
-
         base_dir: **string / default: 'model_dir/'**
             | The ``base_dir`` is where the trained model is saved.
 
@@ -93,13 +90,20 @@ class evaluate():
                 :math:`{f_*, V_c, f_x, \tau, \alpha, \nu_\mathrm{min}}` and
                 :math:`{R_\mathrm{mfp}}` (see the ``globalemu`` paper and
                 references therein for a description of the parameters).
+                You can pass a single set of parameters or a 2D array of
+                different parameters to evaluate. For example if I wanted to
+                evaluate 100 sets of 7 parameters my input array should have
+                shape=(100, 7).
 
     **Return:**
 
         signal: **array or float**
             | The emulated signal. If a single redshift is passed to the
                 emulator then the returned signal will be a single float
-                otherwise the result will be an array.
+                otherwise the result will be an array. If more than
+                one set of parameters are input then the output signal will
+                be an array of signals. e.g. 100 input sets of parameters
+                gives signal.shape=(100, len(z)).
 
         z: **array or float**
             | The redshift values corresponding to the returned signal. If
@@ -113,10 +117,8 @@ class evaluate():
 
         for key, values in kwargs.items():
             if key not in set(
-                    ['xHI', 'base_dir', 'model', 'logs', 'gc', 'z']):
-                raise KeyError("Unexpected keyward argument in evaluate()")
-
-        self.xHI = kwargs.pop('xHI', False)
+                    ['base_dir', 'model', 'logs', 'gc', 'z']):
+                raise KeyError("Unexpected keyword argument in evaluate()")
 
         self.base_dir = kwargs.pop('base_dir', 'model_dir/')
         if type(self.base_dir) is not str:
@@ -124,9 +126,13 @@ class evaluate():
         elif self.base_dir.endswith('/') is False:
             raise KeyError("'base_dir' must end with '/'.")
 
+        file = open(self.base_dir + "preprocess_settings.pkl", "rb")
+        self.preprocess_settings = pickle.load(file)
+
         self.data_mins = np.loadtxt(self.base_dir + 'data_mins.txt')
         self.data_maxs = np.loadtxt(self.base_dir + 'data_maxs.txt')
-        self.cdf = np.loadtxt(self.base_dir + 'cdf.txt')
+        if self.preprocess_settings['resampling'] is True:
+            self.cdf = np.loadtxt(self.base_dir + 'cdf.txt')
 
         self.model = kwargs.pop('model', None)
         if self.model is None:
@@ -139,14 +145,15 @@ class evaluate():
             raise TypeError("'logs' must be a list.")
         self.garbage_collection = kwargs.pop('gc', False)
 
-        boolean_kwargs = [self.garbage_collection, self.xHI]
-        boolean_strings = ['gc', 'xHI']
+        boolean_kwargs = [self.garbage_collection]
+        boolean_strings = ['gc']
         for i in range(len(boolean_kwargs)):
             if type(boolean_kwargs[i]) is not bool:
                 raise TypeError("'" + boolean_strings[i] + "' must be a bool.")
 
-        if self.xHI is False:
+        if self.preprocess_settings['AFB'] is True:
             self.AFB = np.loadtxt(self.base_dir + 'AFB.txt')
+        if self.preprocess_settings['std_division'] is True:
             self.label_stds = np.load(self.base_dir + 'labels_stds.npy')
 
         self.original_z = np.loadtxt(self.base_dir + 'z.txt')
@@ -159,29 +166,65 @@ class evaluate():
 
         if type(parameters) not in set([np.ndarray, list]):
             raise TypeError("'params' must be a list or np.array.")
+        if type(parameters) is list:
+            parameters = np.array(parameters)
 
-        params = []
-        for i in range(len(parameters)):
-            if i in set(self.logs):
-                if parameters[i] == 0:
-                    parameters[i] = 1e-6
-                params.append(np.log10(parameters[i]))
-            else:
-                params.append(parameters[i])
-
-        normalised_params = [
-            (params[i] - self.data_mins[i]) /
-            (self.data_maxs[i] - self.data_mins[i])
-            for i in range(len(params))]
-
-        norm_z = np.interp(self.z, self.original_z, self.cdf)
+        if len(parameters.shape) == 1:
+            params = []
+            for i in range(len(parameters)):
+                if i in set(self.logs):
+                    if parameters[i] == 0:
+                        parameters[i] = 1e-6
+                    params.append(np.log10(parameters[i]))
+                else:
+                    params.append(parameters[i])
+            normalised_params = np.array([
+                (params[i] - self.data_mins[i]) /
+                (self.data_maxs[i] - self.data_mins[i])
+                for i in range(len(params))])
+        else:
+            params = []
+            for i in range(len(parameters)):
+                params_set = []
+                for j in range(len(parameters[i])):
+                    if j in set(self.logs):
+                        if parameters[i, j] == 0:
+                            parameters[i, j] = 1e-6
+                        params_set.append(np.log10(parameters[i, j]))
+                    else:
+                        params_set.append(parameters[i, j])
+                params.append(params_set)
+            params = np.array(params)
+            normalised_params = np.array([
+                (params[:, i] - self.data_mins[i]) /
+                (self.data_maxs[i] - self.data_mins[i])
+                for i in range(params.shape[1])]).T
 
+        if self.preprocess_settings['resampling'] is True:
+            norm_z = np.interp(self.z, self.original_z, self.cdf)
+        else:
+            norm_z = (self.z - self.original_z.min()) / \
+                     (self.original_z.max() - self.original_z.min())
         if isinstance(norm_z, np.ndarray):
-            x = np.tile(normalised_params, (len(norm_z), 1))
-            x = np.hstack([x, norm_z[:, np.newaxis]])
+            if len(normalised_params.shape) == 1:
+                x = np.tile(normalised_params, (len(norm_z), 1))
+                x = np.hstack([x, norm_z[:, np.newaxis]])
+            else:
+                x = np.hstack([
+                               np.vstack([np.tile(normalised_params[i],
+                                         (len(norm_z), 1))
+                                         for i in
+                                         range(len(normalised_params))]),
+                               np.vstack([norm_z[:, np.newaxis]] *
+                                         normalised_params.shape[0])])
             tensor = tf.convert_to_tensor(x, dtype=tf.float32)
             result = self.model(tensor, training=False).numpy()
-            evaluation = result.T[0]
+            if len(normalised_params.shape) != 1:
+                evaluation = np.array([
+                    result[i:i + len(self.z)]
+                    for i in range(0, len(result), len(self.z))])[:, :, 0]
+            else:
+                evaluation = result.T[0]
             if self.garbage_collection is True:
                 K.clear_session()
                 gc.collect()
@@ -190,7 +233,7 @@ class evaluate():
             result = self.model(x[np.newaxis, :], training=False).numpy()
             evaluation = result[0][0]
 
-        if self.xHI is False:
+        if self.preprocess_settings['std_division'] is True:
             if isinstance(evaluation, np.ndarray):
                 evaluation = [
                     evaluation[i]*self.label_stds
@@ -198,6 +241,10 @@ class evaluate():
             else:
                 evaluation *= self.label_stds
 
+        if self.preprocess_settings['AFB'] is True:
             evaluation += np.interp(self.z, self.original_z, self.AFB)
 
+        if type(evaluation) is not np.ndarray:
+            evaluation = np.array(evaluation)
+        
         return evaluation, self.z
